@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 """
-    :copyright: Copyright 2013 by Łukasz Mierzwa
+    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
     :contact: l.mierzwa@gmail.com
 """
 
@@ -11,14 +11,15 @@ from django import forms
 
 from mongoforms import MongoForm
 
+from mongoengine.django.auth import make_password
+
 from upaas_admin.common.forms import ContribFormFieldGenerator
 from upaas_admin.apps.users.models import User
 from upaas_admin.apps.servers.models import RouterServer, BackendServer
 from upaas_admin.apps.scheduler.models import UserLimits
-from upaas_admin.common.hacks.mongoengine_auth import make_password
 
 
-#FIXME patch crispy form-horizontal to support checkboxes and move to crispy
+# FIXME patch crispy form-horizontal to support checkboxes and move to crispy
 class AdminCreateUserForm(MongoForm):
 
     class Meta:
