@@ -1,10 +1,9 @@
 # -*- coding: utf-8 -*-
 """
-    :copyright: Copyright 2013 by Łukasz Mierzwa
+    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
     :contact: l.mierzwa@gmail.com
 """
 
-
 from __future__ import unicode_literals
 
 import os
@@ -17,8 +16,8 @@ import re
 from copy import deepcopy
 
 from mongoengine import (Document, DateTimeField, StringField, LongField,
-                         ReferenceField, ListField, QuerySetManager,
-                         BooleanField, NULLIFY, signals)
+                         ReferenceField, ListField, DictField, QuerySetManager,
+                         BooleanField, IntField, NULLIFY, signals)
 
 from django.utils.translation import ugettext_lazy as _
 from django.core.urlresolvers import reverse
@@ -33,13 +32,16 @@ from upaas.storage.utils import find_storage_handler
 from upaas.storage.exceptions import StorageError
 from upaas import processes
 
-from upaas_admin.apps.servers.models import RouterServer
+from upaas_admin.apps.servers.models import RouterServer, BackendServer
 from upaas_admin.apps.scheduler.models import ApplicationRunPlan
 from upaas_admin.apps.applications.exceptions import UnpackError
 from upaas_admin.apps.scheduler.base import select_best_backends
+from upaas_admin.apps.tasks.constants import TaskStatus
 from upaas_admin.apps.tasks.models import Task
-from upaas_admin.apps.tasks.base import VirtualTask
-from upaas_admin.apps.tasks.constants import TaskStatus, ACTIVE_TASK_STATUSES
+from upaas_admin.apps.applications.constants import (
+    NeedsBuildingFlag, NeedsStoppingFlag, NeedsRestartFlag, IsStartingFlag,
+    NeedsUpgradeFlag, FLAGS_BY_NAME)
+from upaas_admin.apps.applications.helpers import ApplicationStateHelper
 
 
 log = logging.getLogger(__name__)
@@ -49,11 +51,13 @@ class Package(Document):
     date_created = DateTimeField(required=True, default=datetime.datetime.now)
     metadata = StringField(help_text=_('Application metadata'))
     application = ReferenceField('Application', dbref=False, required=True)
+    task = ReferenceField(Task, dbref=False)
 
     interpreter_name = StringField(required=True)
     interpreter_version = StringField(required=True)
 
     parent = StringField()
+    parent_package = ReferenceField('Package')
     filename = StringField()
     bytes = LongField(required=True)
     checksum = StringField(required=True)
@@ -63,6 +67,14 @@ class Package(Document):
     distro_version = StringField(required=True)
     distro_arch = StringField(required=True)
 
+    revision_id = StringField()
+    revision_author = StringField()
+    revision_date = DateTimeField()
+    revision_description = StringField()
+    revision_changelog = StringField()
+
+    ack_filename = '.upaas-unpacked'
+
     meta = {
         'indexes': ['filename'],
         'ordering': ['-date_created'],
@@ -74,6 +86,8 @@ class Package(Document):
     def pre_delete(cls, sender, document, **kwargs):
         log.debug(_("Pre delete signal on package {id}").format(
             id=document.safe_id))
+        Application.objects(id=document.application.id).update_one(
+            pull__packages=document.id)
         document.delete_package_file(null_filename=False)
 
     @property
@@ -97,6 +111,10 @@ class Package(Document):
         """
         return os.path.join(settings.UPAAS_CONFIG.paths.apps, self.safe_id)
 
+    @property
+    def ack_path(self):
+        return os.path.join(self.package_path, self.ack_filename)
+
     def delete_package_file(self, null_filename=True):
         log.debug(_("Deleting package file for {pkg}").format(
             pkg=self.safe_id))
@@ -105,7 +123,8 @@ class Package(Document):
                         "delete").format(pkg=self.safe_id))
             return
 
-        storage = find_storage_handler(self.upaas_config)
+        storage = find_storage_handler(self.upaas_config.storage.handler,
+                                       self.upaas_config.storage.settings)
         if not storage:
             log.error(_("Storage handler '{handler}' not found, cannot "
                         "package file").format(
@@ -140,8 +159,8 @@ class Package(Document):
                     opt_name = opt.split('=')[0].rstrip(' ')
                     if regexp.match(opt_name):
                         options.append(opt)
-                        log.info(_("Adding safe uWSGI option from metadata: "
-                                   "{opt}").format(opt=opt))
+                        log.debug(_("Adding safe uWSGI option from metadata: "
+                                    "{opt}").format(opt=opt))
                         break
 
         return options
@@ -153,7 +172,7 @@ class Package(Document):
         """
 
         def _load_template(path):
-            log.info("Loading uWSGI template from: %s" % path)
+            log.debug("Loading uWSGI template from: %s" % path)
             for search_path in UPAAS_CONFIG_DIRS:
                 template_path = os.path.join(search_path, path)
                 if os.path.exists(template_path):
@@ -199,6 +218,7 @@ class Package(Document):
             'max_workers': backend_conf.workers_max,
             'max_memory': max_memory,
             'memory_per_worker': run_plan.memory_per_worker,
+            'max_log_size': run_plan.max_log_size * 1024 * 1024,
         }
 
         if config.apps.graphite.carbon:
@@ -280,6 +300,11 @@ class Package(Document):
         options.append('\n# starting ENV variables list')
         for key, value in list(envs.items()):
             options.append('env = %s=%s' % (key, value))
+        options.append(
+            'env = UPAAS_SYSTEM_DOMAIN=%s' % self.application.system_domain)
+        if self.application.custom_domains:
+            options.append('env = UPAAS_CUSTOM_DOMAINS=%s' % ','.join(
+                [d.name for d in self.application.custom_domains]))
 
         options.append('\n# starting options from app metadata')
         for opt in self.uwsgi_options_from_metadata():
@@ -317,70 +342,83 @@ class Package(Document):
         options.append('\n')
         return options
 
-    def save_vassal_config(self, backend):
-        log.info("Generating uWSGI vassal configuration")
-        options = "\n".join(self.generate_uwsgi_config(backend))
-
+    def check_vassal_config(self, options):
+        """
+        Verify is there is uWSGI vassal configuration file and if it doesn't
+         need updating.
+        """
         if os.path.exists(self.application.vassal_path):
             current_hash = calculate_file_sha256(self.application.vassal_path)
             new_hash = calculate_string_sha256(options)
             if current_hash == new_hash:
-                log.info("Vassal is present and valid, skipping rewrite")
-                return
+                return True
+        return False
 
-        log.info("Saving vassal configuration to "
-                 "'%s'" % self.application.vassal_path)
+    def save_vassal_config(self, backend):
+        log.info(_("Generating uWSGI vassal configuration"))
+        options = "\n".join(self.generate_uwsgi_config(backend))
+
+        if self.check_vassal_config(options):
+            log.info("Vassal is present and valid, skipping rewrite")
+            return
+
+        log.info(_("Saving vassal configuration to {path}").format(
+            path=self.application.vassal_path))
         with open(self.application.vassal_path, 'w') as vassal:
             vassal.write(options)
-        log.info("Vassal saved")
+        log.info(_("Vassal saved"))
 
     def unpack(self):
-        upaas_config = self.upaas_config
-
         # directory is encoded into string to prevent unicode errors
-        directory = tempfile.mkdtemp(dir=upaas_config.paths.workdir,
+        directory = tempfile.mkdtemp(dir=self.upaas_config.paths.workdir,
                                      prefix="upaas_package_").encode("utf-8")
 
-        storage = find_storage_handler(upaas_config)
+        storage = find_storage_handler(self.upaas_config.storage.handler,
+                                       self.upaas_config.storage.settings)
         if not storage:
             log.error("Storage handler '%s' not "
-                      "found" % upaas_config.storage.handler)
+                      "found" % self.upaas_config.storage.handler)
 
         workdir = os.path.join(directory, "system")
         pkg_path = os.path.join(directory, self.filename)
 
         if os.path.exists(self.package_path):
-            log.error("Package directory already exists: "
-                      "%s" % self.package_path)
-            raise UnpackError("Package directory already exists")
+            log.error(_("Package directory already exists: {path}").format(
+                path=self.package_path))
+            raise UnpackError(_("Package directory already exists"))
 
         log.info("Fetching package '%s'" % self.filename)
         try:
             storage.get(self.filename, pkg_path)
         except StorageError:
-            log.error("Storage error while fetching package "
-                      "'%s'" % self.filename)
+            log.error(_("Storage error while fetching package {name}").format(
+                name=self.filename))
             utils.rmdirs(directory)
-            raise StorageError("Can't fetch package '%s' from "
-                               "storage" % self.filename)
+            raise UnpackError(_("Storage error while fetching package "
+                                "{name}").format(name=self.filename))
 
         log.info("Unpacking package")
         os.mkdir(workdir, 0o755)
         if not tar.unpack_tar(pkg_path, workdir):
-            log.error("Error while unpacking package to '%s'" % workdir)
+            log.error(_("Error while unpacking package to '{workdir}'").format(
+                workdir=workdir))
             utils.rmdirs(directory)
-            raise UnpackError("Error during package unpack")
+            raise UnpackError(_("Error during package unpack"))
 
-        log.info("Package unpacked, moving into '%s'" % self.package_path)
+        with open(os.path.join(workdir, self.ack_filename), 'w') as ack:
+            ack.write(_('Unpacked: {now}').format(now=datetime.datetime.now()))
+
+        log.info(_("Package unpacked, moving into '{path}'").format(
+            path=self.package_path))
         try:
             shutil.move(workdir, self.package_path)
         except shutil.Error as e:
-            log.error("Error while moving unpacked package to final "
-                      "destination: %s" % e)
+            log.error(_("Error while moving unpacked package to final "
+                        "destination: e").format(e=e))
             utils.rmdirs(directory, self.package_path)
-            raise UnpackError("Can't move to final directory "
-                              "'%s'" % self.package_path)
-        log.info("Package moved")
+            raise UnpackError(_("Can't move to final directory: "
+                                "{path}").format(path=self.package_path))
+        log.info(_("Package moved"))
         utils.rmdirs(directory)
 
 
@@ -395,13 +433,50 @@ class ApplicationDomain(Document):
         return str(self.id)
 
 
+class FlagLock(Document):
+    date_created = DateTimeField(required=True, default=datetime.datetime.now)
+    application = ReferenceField('Application', dbref=False, required=True)
+    flag = StringField(required=True)
+    backend = ReferenceField(BackendServer, reverse_delete_rule=NULLIFY)
+    pid = IntField(required=True)
+
+    meta = {
+        'indexes': [
+            {'fields': ['application', 'flag', 'backend'], 'unique': True},
+        ],
+        'ordering': ['-date_created'],
+    }
+
+
+class ApplicationFlag(Document):
+    date_created = DateTimeField(required=True, default=datetime.datetime.now)
+    application = ReferenceField('Application', dbref=False, required=True)
+    name = StringField(required=True, unique_with='application')
+    options = DictField()
+    pending = BooleanField(default=True)
+    pending_backends = ListField(ReferenceField(BackendServer))
+
+    meta = {
+        'indexes': [
+            {'fields': ['name', 'application'], 'unique': True},
+            # TODO add indexes after profiling
+        ],
+        'ordering': ['-date_created'],
+    }
+
+    @property
+    def title(self):
+        return FLAGS_BY_NAME.get(self.name).title
+
+
 class Application(Document):
     date_created = DateTimeField(required=True, default=datetime.datetime.now)
     name = StringField(required=True, min_length=2, max_length=60,
                        unique_with='owner', verbose_name=_('name'))
     # FIXME reverse_delete_rule=DENY for owner
     owner = ReferenceField('User', dbref=False, required=True)
-    metadata = StringField(verbose_name=_('Application metadata'))
+    metadata = StringField(verbose_name=_('Application metadata'),
+                           required=True)
     current_package = ReferenceField(Package, dbref=False, required=False)
     packages = ListField(ReferenceField(Package, dbref=False,
                                         reverse_delete_rule=NULLIFY))
@@ -411,11 +486,15 @@ class Application(Document):
     meta = {
         'indexes': [
             {'fields': ['name', 'owner'], 'unique': True},
-            {'fields': ['packages']}
+            {'fields': ['packages']},
         ],
         'ordering': ['name'],
     }
 
+    def __init__(self, *args, **kwargs):
+        super(Application, self).__init__(*args, **kwargs)
+        self.state_helper = ApplicationStateHelper(self)
+
     @property
     def safe_id(self):
         return str(self.id)
@@ -466,6 +545,15 @@ class Application(Document):
             return utils.select_best_version(self.upaas_config,
                                              self.metadata_config)
 
+    @property
+    def supported_interpreter_versions(self):
+        """
+        Return list of interpreter versions that this app can run.
+        """
+        if self.metadata:
+            return sorted(list(utils.supported_versions(
+                self.upaas_config, self.metadata_config).keys()), reverse=True)
+
     @property
     def run_plan(self):
         """
@@ -485,21 +573,7 @@ class Application(Document):
         """
         List of all tasks for this application.
         """
-        return Task.find('ApplicationTask', application=self)
-
-    @property
-    def active_tasks(self):
-        """
-        List of all active (pending or running) application tasks.
-        """
-        return self.tasks.filter(status__in=ACTIVE_TASK_STATUSES)
-
-    @property
-    def pending_tasks(self):
-        """
-        List of all pending tasks for this application.
-        """
-        return self.tasks.filter(status=TaskStatus.pending)
+        return Task.objects(application=self)
 
     @property
     def running_tasks(self):
@@ -513,22 +587,7 @@ class Application(Document):
         """
         List of all build tasks for this application.
         """
-        return Task.find('BuildPackageTask', application=self)
-
-    @property
-    def active_build_tasks(self):
-        """
-        List of all active (pending or running) build tasks for this
-        application.
-        """
-        return self.build_tasks.filter(status__in=ACTIVE_TASK_STATUSES)
-
-    @property
-    def pending_build_tasks(self):
-        """
-        List of pending build tasks for this application.
-        """
-        return self.build_tasks.filter(status=TaskStatus.pending)
+        return self.tasks.filter(flag=NeedsBuildingFlag.name)
 
     @property
     def running_build_tasks(self):
@@ -537,6 +596,13 @@ class Application(Document):
         """
         return self.build_tasks.filter(status=TaskStatus.running)
 
+    @property
+    def flags(self):
+        """
+        Return list of application flags.
+        """
+        return ApplicationFlag.objects(application=self)
+
     @property
     def system_domain(self):
         """
@@ -565,18 +631,20 @@ class Application(Document):
     def get_absolute_url(self):
         return reverse('app_details', args=[self.safe_id])
 
-    def build_package(self, force_fresh=False):
-        if self.pending_build_tasks:
-            log.info(_("Application {name} is already queued for "
-                       "building").format(name=self.name))
-        else:
-            task = Task.put('BuildPackageTask', application=self,
-                            metadata=self.metadata,
-                            force_fresh=force_fresh)
-            return task
+    def build_package(self, force_fresh=False, interpreter_version=None):
+        q = {
+            'set__options__{0:s}'.format(
+                NeedsBuildingFlag.Options.build_fresh_package): force_fresh,
+            'set__options__{0:s}'.format(
+                NeedsBuildingFlag.Options.build_interpreter_version):
+                    interpreter_version,
+            'unset__pending': True,
+            'upsert': True
+        }
+        ApplicationFlag.objects(application=self,
+                                name=NeedsBuildingFlag.name).update_one(**q)
 
     def start_application(self):
-        #FIXME check if application can start (running apps limit)
         if self.current_package:
             run_plan = self.run_plan
             if not run_plan:
@@ -591,28 +659,15 @@ class Application(Document):
                 run_plan.delete()
                 return
 
-            kwargs = {}
-            vtask = None
-            if len(backends) > 1:
-                vtask = VirtualTask(
-                    title=_("Starting application {name}").format(
-                        name=self.name))
-                vtask.save()
-                kwargs['parent'] = vtask
-
             run_plan.backends = backends
             run_plan.save()
+            ApplicationFlag.objects(
+                application=self, name=IsStartingFlag.name).update_one(
+                    set__pending_backends=[b.backend for b in backends],
+                    upsert=True)
 
-            tasks = []
-            for backend_conf in backends:
-                log.info(_("Set backend '{backend}' in '{name}' run "
-                           "plan").format(backend=backend_conf.backend.name,
-                                          name=self.name))
-                tasks.append(Task.put('StartApplicationTask',
-                                      backend=backend_conf.backend,
-                                      application=self, limit=1, **kwargs))
-            if vtask and len([_f for _f in tasks if _f]) == 0:
-                vtask.delete()
+            # FIXME what if there are waiting stop tasks on other backends ?
+            self.flags.filter(name=NeedsStoppingFlag.name).delete()
 
     def stop_application(self):
         if self.current_package:
@@ -622,45 +677,30 @@ class Application(Document):
                 # no backends in run plan, just delete it
                 self.run_plan.delete()
                 return
+        ApplicationFlag.objects(
+            application=self, name=NeedsStoppingFlag.name).update_one(
+                set__pending_backends=[
+                    b.backend for b in self.run_plan.backends], upsert=True)
+        self.flags.filter(
+            name__in=[IsStartingFlag.name, NeedsRestartFlag.name]).delete()
 
-            kwargs = {}
-            vtask = None
-            if len(self.run_plan.backends) > 1:
-                vtask = VirtualTask(
-                    title=_("Stopping application {name}").format(
-                        name=self.name))
-                vtask.save()
-                kwargs['parent'] = vtask
-
-            tasks = []
-            for backend_conf in self.run_plan.backends:
-                tasks.append(Task.put('StopApplicationTask',
-                                      backend=backend_conf.backend,
-                                      application=self, limit=1, **kwargs))
-            if vtask and len([_f for _f in tasks if _f]) == 0:
-                vtask.delete()
+    def restart_application(self):
+        if self.current_package:
+            if not self.run_plan:
+                return
+        ApplicationFlag.objects(
+            application=self, name=NeedsRestartFlag.name).update_one(
+                set__pending_backends=[
+                    b.backend for b in self.run_plan.backends], upsert=True)
 
     def upgrade_application(self):
         if self.current_package:
             if not self.run_plan:
                 return
-
-            kwargs = {}
-            vtask = None
-            if len(self.run_plan.backends) > 1:
-                vtask = VirtualTask(
-                    title=_("Upgrading application {name}").format(
-                        name=self.name))
-                vtask.save()
-                kwargs['parent'] = vtask
-
-            tasks = []
-            for backend_conf in self.run_plan.backends:
-                tasks.append(Task.put('UpgradeApplicationTask',
-                                      backend=backend_conf.backend,
-                                      application=self, limit=1, **kwargs))
-            if vtask and len([_f for _f in tasks if _f]) == 0:
-                vtask.delete()
+        ApplicationFlag.objects(
+            application=self, name=NeedsUpgradeFlag.name).update_one(
+                set__pending_backends=[
+                    b.backend for b in self.run_plan.backends], upsert=True)
 
     def update_application(self):
         if self.run_plan:
@@ -674,26 +714,14 @@ class Application(Document):
                             "available").format(name=self.name))
                 return
 
-            kwargs = {}
-            vtask = None
-            if len(current_backends) > 1 or len(new_backends) > 1:
-                vtask = VirtualTask(
-                    title=_("Updating application {name}").format(
-                        name=self.name))
-                vtask.save()
-                kwargs['parent'] = vtask
-
-            tasks = []
+            updated_backends = []
             for backend_conf in new_backends:
                 if backend_conf.backend in current_backends:
                     # replace backend settings with updated version
                     run_plan.update(
                         pull__backends__backend=backend_conf.backend)
                     run_plan.update(push__backends=backend_conf)
-                    tasks.append(Task.put('UpdateVassalTask',
-                                          backend=backend_conf.backend,
-                                          application=self,
-                                          limit=1, **kwargs))
+                    updated_backends.append(backend_conf.backend)
                 else:
                     # add backend to run plan if not already there
                     ApplicationRunPlan.objects(
@@ -701,21 +729,30 @@ class Application(Document):
                         backends__backend__nin=[
                             backend_conf.backend]).update_one(
                         push__backends=backend_conf)
-                    tasks.append(Task.put('StartApplicationTask',
-                                          backend=backend_conf.backend,
-                                          application=self, limit=1, **kwargs))
+                    log.info(_("Starting {name} on backend {backend}").format(
+                        name=self.name, backend=backend_conf.backend.name))
+                    ApplicationFlag.objects(
+                        pending_backends__ne=backend_conf.backend,
+                        application=self,
+                        name=IsStartingFlag.name).update_one(
+                            add_to_set__pending_backends=backend_conf.backend,
+                            upsert=True)
+
+            if updated_backends:
+                ApplicationFlag.objects(
+                    application=self, name=NeedsRestartFlag.name).update_one(
+                        set__pending_backends=updated_backends, upsert=True)
 
             for backend in current_backends:
                 if backend not in [bc.backend for bc in new_backends]:
                     log.info(_("Stopping {name} on old backend "
                                "{backend}").format(name=self.name,
                                                    backend=backend.name))
-                    tasks.append(Task.put('StopApplicationTask',
-                                          backend=backend, application=self,
-                                          **kwargs))
-
-            if vtask and len([_f for _f in tasks if _f]) == 0:
-                vtask.delete()
+                    ApplicationFlag.objects(
+                        pending_backends__ne=backend,
+                        application=self,
+                        name=NeedsStoppingFlag.name).update_one(
+                            add_to_set__pending_backends=backend, upsert=True)
 
     def trim_package_files(self):
         """
@@ -723,7 +760,8 @@ class Application(Document):
         app that are kept in database for rollback feature are set in user
         limits as 'packages_per_app'.
         """
-        storage = find_storage_handler(self.upaas_config)
+        storage = find_storage_handler(self.upaas_config.storage.handler,
+                                       self.upaas_config.storage.settings)
         if not storage:
             log.error("Storage handler '%s' not found, cannot trim "
                       "packages" % self.upaas_config.storage.handler)
