@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 """
-    :copyright: Copyright 2013 by Łukasz Mierzwa
+    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
     :contact: l.mierzwa@gmail.com
 """
 
@@ -225,7 +225,6 @@ class ApplicationTaskDetailsView(LoginRequiredMixin, OwnedAppTasksMixin,
     template_name = 'task_details.html'
     slug_field = 'id'
     context_object_name = 'task'
-    #TODO add support for virtual tasks
 
 
 class UpdateApplicationMetadataView(LoginRequiredMixin, OwnedAppsMixin,
@@ -298,6 +297,7 @@ class StartApplicationView(LoginRequiredMixin, OwnedAppsMixin,
         form.instance.application = self.app
         form.instance.memory_per_worker = self.request.user.limits[
             'memory_per_worker']
+        form.instance.max_log_size = self.request.user.limits['max_log_size']
         ret = super(StartApplicationView, self).form_valid(form)
         self.app.start_application()
         return ret
@@ -359,6 +359,7 @@ class EditApplicationRunPlanView(LoginRequiredMixin, OwnedAppsMixin,
     def form_valid(self, form):
         form.instance.memory_per_worker = self.request.user.limits[
             'memory_per_worker']
+        form.instance.max_log_size = self.request.user.limits['max_log_size']
         ret = super(EditApplicationRunPlanView, self).form_valid(form)
         self.app.update_application()
         return ret
@@ -434,10 +435,6 @@ class PackageDeleteView(LoginRequiredMixin, OwnedPackagesMixin,
     def form_valid(self, form):
         self.get_context_data()
         if self.object.id != self.object.application.current_package.id:
-            if self.object.filename:
-                self.object.delete_package_file(null_filename=False)
-            Application.objects(id=self.application.id).update_one(
-                pull__packages=self.object.id)
             self.object.delete()
             return HttpResponseRedirect(self.get_success_url())
         else:
@@ -459,10 +456,28 @@ class BuildPackageView(AppActionView):
             return reverse('app_task_details', args=[self.task.safe_id])
         return reverse('app_details', args=[self.object.safe_id])
 
+    def get_form(self, form_class):
+        form = super(BuildPackageView, self).get_form(form_class)
+        build_types = []
+        if self.object.current_package:
+            build_types.append((
+                '',
+                _('Incremental package, using {interpreter} {ver}').format(
+                    interpreter=self.object.interpreter_name,
+                    ver=self.object.interpreter_version)))
+        for ver in self.object.supported_interpreter_versions:
+            build_types.append((
+                ver,
+                _('New fresh package, using {interpreter} {ver}').format(
+                    interpreter=self.object.interpreter_name, ver=ver)))
+        form.fields['build_type'].choices = build_types
+        return form
+
     def action(self, form):
-        if self.object and self.object.metadata:
-            self.task = self.object.build_package(
-                force_fresh=form.cleaned_data['force_fresh'])
+        if self.object:
+            self.object.build_package(
+                force_fresh=form.cleaned_data['build_type'] != '',
+                interpreter_version=form.cleaned_data['build_type'] or None)
 
 
 class RollbackApplicationView(OwnedPackagesMixin, AppActionView):
