@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 """
-    :copyright: Copyright 2013 by Łukasz Mierzwa
+    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
     :contact: l.mierzwa@gmail.com
 """
 
@@ -18,7 +18,7 @@ class ApplicationRunPlanForm(CrispyMongoForm):
 
     submit_label = 'Start'
     submit_icon_class = 'fa fa-play'
-    #FIXME patch crispy form-horizontal to support checkboxes
+    # FIXME patch crispy form-horizontal to support checkboxes
     form_class = ''
     label_class = ''
     field_class = ''
@@ -26,7 +26,8 @@ class ApplicationRunPlanForm(CrispyMongoForm):
 
     class Meta:
         document = ApplicationRunPlan
-        exclude = ('application', 'backends', 'memory_per_worker')
+        exclude = ('application', 'backends', 'memory_per_worker',
+                   'max_log_size')
 
     def clean(self):
         workers_min = self.cleaned_data.get('workers_min')
@@ -75,4 +76,5 @@ class EditApplicationRunPlanForm(ApplicationRunPlanForm):
 
     class Meta:
         document = ApplicationRunPlan
-        exclude = ('application', 'backends', 'memory_per_worker')
+        exclude = ('application', 'backends', 'memory_per_worker',
+                   'max_log_size')
