@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 """
-    :copyright: Copyright 2013 by Łukasz Mierzwa
+    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
     :contact: l.mierzwa@gmail.com
 """
 
@@ -12,7 +12,7 @@ import logging
 from tastypie_mongoengine.resources import MongoEngineResource
 
 from tastypie.resources import ALL
-from tastypie.authorization import Authorization
+from tastypie.authorization import ReadOnlyAuthorization
 
 from upaas_admin.apps.servers.models import BackendServer, RouterServer
 from upaas_admin.common.apiauth import UpaasApiKeyAuthentication
@@ -24,6 +24,7 @@ log = logging.getLogger(__name__)
 class BackendResource(MongoEngineResource):
 
     class Meta:
+        always_return_data = True
         queryset = BackendServer.objects.all()
         resource_name = 'backend'
         filtering = {
@@ -33,12 +34,13 @@ class BackendResource(MongoEngineResource):
         }
         always_return_data = True
         authentication = UpaasApiKeyAuthentication()
-        authorization = Authorization()
+        authorization = ReadOnlyAuthorization()
 
 
 class RouterResource(MongoEngineResource):
 
     class Meta:
+        always_return_data = True
         queryset = RouterServer.objects.all()
         resource_name = 'router'
         filtering = {
@@ -48,4 +50,4 @@ class RouterResource(MongoEngineResource):
             'name': ALL,
         }
         authentication = UpaasApiKeyAuthentication()
-        authorization = Authorization()
+        authorization = ReadOnlyAuthorization()
