@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 """
-    :copyright: Copyright 2013 by Łukasz Mierzwa
+    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
     :contact: l.mierzwa@gmail.com
 """
 
@@ -50,6 +50,10 @@ class BackendServer(Document):
             if backend:
                 return backend
 
+    @property
+    def safe_id(self):
+        return str(self.id)
+
     @property
     def run_plans(self):
         """
@@ -116,8 +120,8 @@ class BackendServer(Document):
                       "ports" % self.maximum_ports)
             return
         while True:
-            #TODO random can take very long to find free port if port usage is
-            #TODO high, maybe linear search would be better?
+            # TODO random can take very long to find free port if port usage is
+            # TODO high, maybe linear search would be better?
             port = randrange(self.port_min, self.port_max+1)
             if port not in self.allocated_ports and port not in ports:
                 ports.append(port)
@@ -148,3 +152,7 @@ class RouterServer(Document):
         'indexes': ['name', 'private_ip', 'public_ip', 'is_enabled'],
         'ordering': ['name'],
     }
+
+    @property
+    def safe_id(self):
+        return str(self.id)
