@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 """
-    :copyright: Copyright 2013 by Łukasz Mierzwa
+    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
     :contact: l.mierzwa@gmail.com
 """
 
@@ -15,10 +15,16 @@ from IPy import IP
 DEBUG = True
 TEMPLATE_DEBUG = True
 
+LOGGING['formatters']['standard']['format'] = "[pid:%(process)s] " \
+                                              "%(levelname)s [%(name)s:" \
+                                              "%(lineno)s] %(message)s"
 
-#==============================================================================
+EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'
+
+
+# =============================================================================
 # django-debug-toolbar
-#==============================================================================
+# =============================================================================
 
 INSTALLED_APPS += (
     'debug_toolbar',
@@ -68,7 +74,6 @@ DEBUG_TOOLBAR_PANELS = (
     'debug_toolbar.panels.redirects.RedirectsPanel',
     'template_timings_panel.panels.TemplateTimings.TemplateTimings',
     'debug_toolbar_mongo.panel.MongoDebugPanel',
-    #'debug_toolbar.panels.logging.LoggingPanel',
 )
 
 DEBUG_TOOLBAR_CONFIG = {
