@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 """
-    :copyright: Copyright 2013 by Łukasz Mierzwa
+    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
     :contact: l.mierzwa@gmail.com
 """
 
@@ -22,9 +22,9 @@ from django.conf.global_settings import *   # pylint: disable=W0614,W0401
 log = logging.basicConfig()
 
 
-#==============================================================================
+# =============================================================================
 # UPaaS configuraton file
-#==============================================================================
+# =============================================================================
 UPAAS_CONFIG = load_main_config()
 
 if not UPAAS_CONFIG:
@@ -32,9 +32,9 @@ if not UPAAS_CONFIG:
     sys.exit(1)
 
 
-#==============================================================================
+# =============================================================================
 # Generic Django project settings
-#==============================================================================
+# =============================================================================
 
 DATABASES = {
     'default': {
@@ -71,7 +71,6 @@ INSTALLED_APPS = (
     'django.contrib.staticfiles',
     'mongoengine.django.mongo_auth',
     'djangojs',
-    'dajaxice',
     'pipeline',
     'crispy_forms',
     'django_bootstrap_breadcrumbs',
@@ -79,6 +78,7 @@ INSTALLED_APPS = (
     'tastypie_mongoengine',
     'pure_pagination',
     'django_gravatar',
+    'passwords',
     'upaas_admin.apps.users',
     'upaas_admin.apps.applications',
     'upaas_admin.apps.servers',
@@ -88,9 +88,9 @@ INSTALLED_APPS = (
 )
 
 
-#==============================================================================
+# =============================================================================
 # Calculation of directories relative to the project module location
-#==============================================================================
+# =============================================================================
 
 import upaas_admin as project_module
 
@@ -102,9 +102,9 @@ if not os.path.exists(VAR_ROOT):
     os.mkdir(VAR_ROOT)
 
 
-#==============================================================================
+# =============================================================================
 # Project URLS and media settings
-#==============================================================================
+# =============================================================================
 
 ROOT_URLCONF = 'upaas_admin.urls'
 
@@ -119,18 +119,9 @@ STATICFILES_DIRS = (
 )
 
 
-#==============================================================================
-# Static files
-#==============================================================================
-
-STATICFILES_FINDERS += (
-    'dajaxice.finders.DajaxiceFinder',
-)
-
-
-#==============================================================================
+# =============================================================================
 # Templates
-#==============================================================================
+# =============================================================================
 
 TEMPLATE_DIRS = (
     os.path.join(PROJECT_DIR, 'templates'),
@@ -149,20 +140,20 @@ TEMPLATE_LOADERS = (
 )
 
 
-#==============================================================================
+# =============================================================================
 # Middleware
-#==============================================================================
+# =============================================================================
 
 MIDDLEWARE_CLASSES += (
 )
 
 
-#==============================================================================
+# =============================================================================
 # Auth / security
-#==============================================================================
+# =============================================================================
 
 AUTHENTICATION_BACKENDS = (
-    'upaas_admin.common.hacks.mongoengine_auth.MongoEngineBackend',
+    'mongoengine.django.auth.MongoEngineBackend',
 )
 
 AUTH_USER_MODEL = 'mongo_auth.MongoUser'
@@ -173,9 +164,9 @@ LOGIN_REDIRECT_URL = '/'
 
 ALLOWED_HOSTS = UPAAS_CONFIG.admin.domains or ['*']
 
-#==============================================================================
+# =============================================================================
 # MongoEngine
-#==============================================================================
+# =============================================================================
 
 SESSION_ENGINE = 'mongoengine.django.sessions'
 SESSION_SERIALIZER = 'mongoengine.django.sessions.BSONSerializer'
@@ -183,6 +174,7 @@ SESSION_SERIALIZER = 'mongoengine.django.sessions.BSONSerializer'
 MONGOENGINE_USER_DOCUMENT = 'upaas_admin.apps.users.models.User'
 
 from mongoengine import connect
+from pymongo import ReadPreference
 
 mongo_opts = {}
 
@@ -193,7 +185,8 @@ if UPAAS_CONFIG.mongodb.get('uri'):
 else:
     # host & port settings
     mongo_opts = {'host': UPAAS_CONFIG.mongodb.host,
-                  'port': UPAAS_CONFIG.mongodb.port}
+                  'port': UPAAS_CONFIG.mongodb.port,
+                  'read_preference': ReadPreference.PRIMARY_PREFERRED}
     if UPAAS_CONFIG.mongodb.get('username'):
         mongo_opts['username'] = UPAAS_CONFIG.mongodb.username
         if UPAAS_CONFIG.mongodb.get('password'):
@@ -202,9 +195,9 @@ else:
     connect(UPAAS_CONFIG.mongodb.database, **mongo_opts)
 
 
-#==============================================================================
+# =============================================================================
 # django-pipeline
-#==============================================================================
+# =============================================================================
 
 # force pipeline to be enabled even if DEBUG=True
 PIPELINE_ENABLED = True
@@ -215,7 +208,7 @@ PIPELINE_COMPILERS = (
     'pipeline.compilers.less.LessCompiler',
 )
 
-#FIXME disabled js compression for now, fix it and re-enable!
+# FIXME disabled js compression for now, fix it and re-enable!
 PIPELINE_JS_COMPRESSOR = None
 
 PIPELINE_CSS = {
@@ -254,6 +247,9 @@ PIPELINE_JS = {
             'underscore/underscore.string.min.js',
             'haml/haml.min.js',
             'tabdrop/bootstrap-tabdrop.js',
+            'backbone/backbone-min.js',
+            'backbone/backbone-tastypie.js',
+            'backbone/backbone.poller.min.js',
             'upaas/js/base/*.js',
         ),
         'output_filename': 'js/base.js',
@@ -286,14 +282,12 @@ PIPELINE_JS = {
 }
 
 
-#==============================================================================
+# =============================================================================
 # logging
-#==============================================================================
+# =============================================================================
 
 ADMINS = (('UPAAS Admin', 'root@localhost'),)
 
-EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'
-
 loglevel = UPAAS_CONFIG.admin.loglevel.upper()
 if not logging.getLevelName(loglevel):
     log.error("Invalid log level name '%s', ignoring" % loglevel)
@@ -304,8 +298,7 @@ LOGGING = {
     'disable_existing_loggers': False,
     'formatters': {
         'standard': {
-            'format': "[%(asctime)s] [pid: %(process)s] %(levelname)s "
-                      "[%(name)s:%(lineno)s] %(message)s",
+            'format': "[pid:%(process)s] %(levelname)s: %(message)s",
             'datefmt': "%d/%b/%Y %H:%M:%S"
         },
     },
@@ -322,19 +315,46 @@ LOGGING = {
             'level': loglevel,
             'propagate': False,
         },
+        'django.request': {
+            'handlers': ['console'],
+            'level': 'ERROR',
+            'propagate': True,
+        },
+        'upaas.config': {
+            'handlers': ['console'],
+            'level': 'INFO',
+            'propagate': False,
+        },
     },
 }
 
-#==============================================================================
+# =============================================================================
+# email
+# =============================================================================
+
+EMAIL_HOST = UPAAS_CONFIG.admin.smtp.host
+EMAIL_PORT = UPAAS_CONFIG.admin.smtp.port
+DEFAULT_FROM_EMAIL = UPAAS_CONFIG.admin.smtp.sender
+
+# auth is optional
+try:
+    EMAIL_HOST_USER = UPAAS_CONFIG.admin.smtp.username
+    EMAIL_HOST_PASSWORD = UPAAS_CONFIG.admin.smtp.password
+except AttributeError:
+    pass
+
+EMAIL_SUBJECT_PREFIX = ''
+
+# =============================================================================
 # gravatar
-#==============================================================================
+# =============================================================================
 
 GRAVATAR_DEFAULT_IMAGE = 'wavatar'
 
 
-#==============================================================================
+# =============================================================================
 # django-pure-pagination
-#==============================================================================
+# =============================================================================
 
 PAGINATION_SETTINGS = {
     'PAGE_RANGE_DISPLAYED': 6,
@@ -342,23 +362,37 @@ PAGINATION_SETTINGS = {
 }
 
 
-#==============================================================================
+# =============================================================================
 # django-crispy-forms
-#==============================================================================
+# =============================================================================
 
 CRISPY_TEMPLATE_PACK = 'bootstrap3'
 
 
-#==============================================================================
+# =============================================================================
+# django-passwords
+# =============================================================================
+
+PASSWORD_MIN_LENGTH = 8
+PASSWORD_COMPLEXITY = {
+    "UPPER": 1,
+    "LOWER": 1,
+    "DIGITS": 1,
+    "PUNCTUATION": 0,
+    "NON ASCII": 0,
+    "WORDS": 1
+}
+
+# =============================================================================
 # tests
-#==============================================================================
+# =============================================================================
 
 TEST_RUNNER = 'upaas_admin.common.tests.MongoEngineTestRunner'
 
 
-#==============================================================================
+# =============================================================================
 # DEBUG
-#==============================================================================
+# =============================================================================
 
 if UPAAS_CONFIG.admin.debug:
 
