@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 """
-    :copyright: Copyright 2013 by Łukasz Mierzwa
+    :copyright: Copyright 2013-2014 by Łukasz Mierzwa
     :contact: l.mierzwa@gmail.com
 """
 
@@ -10,9 +10,7 @@ from __future__ import unicode_literals
 from django.conf import settings
 from django.conf.urls import patterns, include, url
 from django.contrib.staticfiles.urls import staticfiles_urlpatterns
-from django.views.generic import RedirectView
-
-from dajaxice.core import dajaxice_autodiscover, dajaxice_config
+from django.views.generic import RedirectView, TemplateView
 
 from tastypie.api import Api
 
@@ -22,9 +20,10 @@ from upaas_admin.apps.servers.api import (BackendResource, RouterResource)
 from upaas_admin.apps.scheduler.api import RunPlanResource
 
 from upaas_admin.apps.applications.views import IndexView
-from upaas_admin.apps.tasks.registry import tasks_autodiscover
 
-from upaas_admin.apps.users.forms import UserPasswordChangeForm
+from upaas_admin.apps.users.forms import (UserPasswordChangeForm,
+                                          UserPasswordSetForm)
+from upaas_admin.apps.tasks.api import TaskResource
 
 
 v1_api = Api(api_name='v1')
@@ -33,11 +32,7 @@ v1_api.register(PackageResource())
 v1_api.register(BackendResource())
 v1_api.register(RouterResource())
 v1_api.register(RunPlanResource())
-
-
-dajaxice_autodiscover()
-
-tasks_autodiscover()
+v1_api.register(TaskResource())
 
 
 handler400 = 'upaas_admin.common.handlers.bad_request'
@@ -63,8 +58,26 @@ urlpatterns = patterns(
         {'template_name': 'users/password_changed.html'},
         name='password_change_done'),
 
+    url(r'^password/reset$', 'django.contrib.auth.views.password_reset',
+        {'template_name': 'users/password_reset.html',
+         'email_template_name': 'users/emails/password_reset.html',
+         'subject_template_name': 'users/emails/password_reset_subject.txt',
+         'post_reset_redirect': 'password_reset_sent'},
+        name='password_reset'),
+    url(r'^password/reset/sent$', TemplateView.as_view(
+        template_name='users/password_reset_sent.html'),
+        name='password_reset_sent'),
+    url(r'^password/reset/confirm/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>.+)/$',
+        'upaas_admin.common.hacks.django_password_reset.'
+        'safe_password_reset_confirm',
+        {'template_name': 'users/password_reset_confirm.html',
+         'set_password_form': UserPasswordSetForm},
+        name='password_reset_confirm'),
+    url(r'^password/reset/done$', TemplateView.as_view(
+        template_name='users/password_reset_complete.html'),
+        name='password_reset_complete'),
+
     url(r'^djs/', include('djangojs.urls')),
-    url(dajaxice_config.dajaxice_url, include('dajaxice.urls')),
 
     (r'^api/', include(v1_api.urls)),
 
